@@ -3,12 +3,13 @@ from __future__ import annotations
 import abc
 from dataclasses import dataclass
 from multiprocessing import get_context
-from typing import TYPE_CHECKING, Callable, Iterable
+from typing import TYPE_CHECKING, Callable, Iterable, Literal
 
 import numpy as np
+import tqdm
 
 from .. import to_speech as mu
-from ..interface import ContentType, IDispachItem, Protocol, SegmentLevel
+from ..interface import ContentType, IDispatchItem, Protocol, SegmentLevel
 from ..utility import compress
 
 if TYPE_CHECKING:
@@ -18,7 +19,7 @@ if TYPE_CHECKING:
 
 
 @dataclass
-class ProtocolSegment(IDispachItem):
+class ProtocolSegment(IDispatchItem):
     """Container for a subset of utterances within a single protocol"""
 
     protocol_name: str
@@ -26,12 +27,12 @@ class ProtocolSegment(IDispachItem):
     who: str
     id: str
     data: str
-    page_number: str
+    page_number: int
     year: int
     u_id: str
     n_utterances: int = 0
     speaker_info: SpeakerInfo = None
-    speaker_hash: str = None
+    speaker_note_id: str = None
     speech_index: int = None
 
     def __len__(self) -> int:
@@ -54,10 +55,19 @@ class ProtocolSegment(IDispachItem):
             'filename': self.filename,
             'n_tokens': self.n_tokens,
             'n_utterances': self.n_utterances,
-            'speaker_hash': self.speaker_hash,
+            'speaker_note_id': self.speaker_note_id,
             'speech_index': self.speech_index,
             'page_number': self.page_number,
-            **({} if not self.speaker_info else self.speaker_info.to_dict()),
+            **(
+                {}
+                if not self.speaker_info
+                else {
+                    'gender_id': self.speaker_info.gender_id,
+                    'party_id': self.speaker_info.party_id,
+                    'office_type_id': self.speaker_info.term_of_office.office_type_id,
+                    'sub_office_type_id': self.speaker_info.term_of_office.sub_office_type_id,
+                }  # FIXME: Call self.speaker_info.to_dict() instead
+            ),
         }
 
     @staticmethod
@@ -83,23 +93,25 @@ class ProtocolSegment(IDispachItem):
         return self.data
 
 
-def to_protocol_segment(*, protocol: Protocol, content_type: ContentType, **_) -> list[ProtocolSegment]:
+def to_protocol_segment(
+    *, protocol: Protocol, content_type: ContentType, which_year: Literal["filename", "date"] = "filename", **_
+) -> list[ProtocolSegment]:
     return [
         ProtocolSegment(
             protocol_name=protocol.name,
             content_type=content_type,
             segment_level=SegmentLevel.Protocol,
-            year=protocol.year,
+            year=protocol.get_year(which=which_year),
             name=protocol.name,
             who=None,
             id=protocol.name,
             u_id=None,
             data=protocol.text,
-            page_number='0',
+            page_number=0,
             n_tokens=0,
             n_utterances=len(protocol.utterances),
             speaker_info=None,
-            speaker_hash=None,
+            speaker_note_id=None,
         )
     ]
 
@@ -110,6 +122,7 @@ def to_speech_segments(
     content_type: ContentType,
     segment_skip_size: int,
     merge_strategy: mu.MergeStrategyType,
+    which_year: Literal["filename", "date"] = "filename",
     **_,
 ) -> list[ProtocolSegment]:
     return [
@@ -117,7 +130,7 @@ def to_speech_segments(
             protocol_name=protocol.name,
             content_type=content_type,
             segment_level=SegmentLevel.Speech,
-            year=protocol.year,
+            year=protocol.get_year(which=which_year),
             name=s.document_name,
             who=s.who,
             id=s.speech_id,
@@ -126,7 +139,7 @@ def to_speech_segments(
             page_number=s.page_number,
             n_tokens=0 if not s.has_tagged_text else s.tagged_text.count("\n"),
             n_utterances=len(s),
-            speaker_hash=s.speaker_hash,
+            speaker_note_id=s.speaker_note_id,
             speech_index=s.speech_index,
         )
         for s in mu.to_speeches(protocol=protocol, merge_strategy=merge_strategy, skip_size=segment_skip_size)
@@ -134,14 +147,19 @@ def to_speech_segments(
 
 
 def to_who_segments(
-    *, protocol: Protocol, content_type: ContentType, segment_skip_size: int, **_
+    *,
+    protocol: Protocol,
+    content_type: ContentType,
+    segment_skip_size: int,
+    which_year: Literal["filename", "date"] = "filename",
+    **_,
 ) -> list[ProtocolSegment]:
     return [
         ProtocolSegment(
             protocol_name=protocol.name,
             content_type=content_type,
             segment_level=SegmentLevel.Who,
-            year=protocol.year,
+            year=protocol.get_year(which=which_year),
             name=s.document_name,
             who=s.who,
             id=s.speech_id,
@@ -155,13 +173,15 @@ def to_who_segments(
     ]
 
 
-def to_utterance_segments(*, protocol: Protocol, content_type: ContentType, **_) -> list[ProtocolSegment]:
+def to_utterance_segments(
+    *, protocol: Protocol, content_type: ContentType, which_year: Literal["filename", "date"] = "filename", **_
+) -> list[ProtocolSegment]:
     return [
         ProtocolSegment(
             protocol_name=protocol.name,
             content_type=content_type,
             segment_level=SegmentLevel.Utterance,
-            year=protocol.year,
+            year=protocol.get_year(which=which_year),
             name=f'{protocol.name}_{i+1:03}',
             who=u.who,
             id=u.u_id,
@@ -175,13 +195,15 @@ def to_utterance_segments(*, protocol: Protocol, content_type: ContentType, **_)
     ]
 
 
-def to_paragraph_segments(*, protocol: Protocol, content_type: ContentType, **_) -> list[ProtocolSegment]:
+def to_paragraph_segments(
+    *, protocol: Protocol, content_type: ContentType, which_year: Literal["filename", "date"] = "filename", **_
+) -> list[ProtocolSegment]:
     return [
         ProtocolSegment(
             protocol_name=protocol.name,
             content_type=content_type,
             segment_level=SegmentLevel.Paragraph,
-            year=protocol.year,
+            year=protocol.get_year(which=which_year),
             name=f'{protocol.name}_{j+1:03}_{i+1:03}',
             who=u.who,
             id=f"{u.u_id}@{i}",
@@ -204,6 +226,7 @@ def to_segments(
     merge_strategy: mu.MergeStrategyType,
     segment_skip_size: int = 1,
     preprocess: Callable[[str], str] = None,
+    which_year: Literal["filename", "date"] = "filename",
 ) -> Iterable[ProtocolSegment]:
     """Splits protocol to sequence of text/tagged text segments
 
@@ -222,6 +245,7 @@ def to_segments(
         content_type=content_type,
         segment_skip_size=segment_skip_size,
         merge_strategy=merge_strategy,
+        which_year=which_year,
     )
 
     if preprocess is not None:
@@ -245,8 +269,6 @@ SEGMENT_FUNCTIONS: dict = {
 
 
 class ProtocolSegmentIterator(abc.ABC):
-    ...
-
     def __init__(
         self,
         *,
@@ -259,6 +281,7 @@ class ProtocolSegmentIterator(abc.ABC):
         multiproc_keep_order: bool = False,
         merge_strategy: str = 'chain',
         preprocess: Callable[[str], str] = None,
+        which_year: Literal["filename", "date"] = "filename",
     ):
         """Merge utterances within protocols to segments.
 
@@ -272,6 +295,7 @@ class ProtocolSegmentIterator(abc.ABC):
             multiproc_keep_order (bool, optional): Keep doc order. Defaults to False.
             merge_strategy (str, optional): Speech merge strategy. Defaults to 'chain'.
             preprocess (Callable[[str], str], optional): Preprocess funcion, only used for text. Defaults to None.
+            which_year (Literal["filename", "date"]): Take year from filename or XML tag `date` in content
         """
         self.filenames: list[str] = sorted(filenames)
         self.iterator = None
@@ -283,6 +307,7 @@ class ProtocolSegmentIterator(abc.ABC):
         self.multiproc_chunksize: int = multiproc_chunksize
         self.multiproc_keep_order: bool = multiproc_keep_order
         self.preprocess: Callable[[ProtocolSegment], str] = preprocess
+        self.which_year: str = which_year
 
     def __iter__(self):
         self.iterator = self.create_iterator()
@@ -292,14 +317,20 @@ class ProtocolSegmentIterator(abc.ABC):
         return next(self.iterator)
 
     def create_iterator(self) -> Iterable[ProtocolSegment]:
-
         item: ProtocolSegment
         fx = self.preprocess
         # speaker_service: SpeakerInfoService = self.speaker_service
 
         if self.multiproc_processes > 1:
             args: list[tuple[str, str, str, int]] = [
-                (name, self.content_type, self.segment_level, self.segment_skip_size, self.merge_strategy)
+                (
+                    name,
+                    self.content_type,
+                    self.segment_level,
+                    self.segment_skip_size,
+                    self.merge_strategy,
+                    self.which_year,
+                )
                 for name in self.filenames
             ]
             with get_context("spawn").Pool(processes=self.multiproc_processes) as executor:
@@ -311,7 +342,7 @@ class ProtocolSegmentIterator(abc.ABC):
                             fx(item)
                         yield item
         else:
-            for filename in self.filenames:
+            for filename in tqdm.tqdm(self.filenames):
                 for item in self.load(filename=filename):
                     if fx:
                         fx(item)
