@@ -11,17 +11,18 @@ This source code is heavily influenced by the source code found at https://gitla
 License: https://gitlab.com/Julipan/swedish-de-hyphenator/-/blob/master/LICENSE
 
 """
-import os
 import re
 from dataclasses import dataclass, field
 from enum import IntEnum
-from typing import Callable, Dict, Set
+from os.path import isfile, join
+from typing import Callable
 
-from ..utility import load_dict, load_token_set, store_dict, store_token_set
+from .utility import load_dict, load_token_set, store_dict, store_token_set
 
 PARAGRAPH_MARKER = '##PARAGRAPH##'
 
 # pylint: disable=too-many-arguments
+jj = join
 
 
 class WhitelistReason(IntEnum):
@@ -31,7 +32,7 @@ class WhitelistReason(IntEnum):
     UnknownParts = 3
 
 
-IGNORE_CONJUNCTION_WORDS: Set[str] = {
+IGNORE_CONJUNCTION_WORDS: set[str] = {
     'än',
     'eller',
     'framför',
@@ -57,82 +58,75 @@ def is_ignored_by_conjunction_word(dashed_word: str) -> bool:
     return any(dashed_word.endswith(f' {w}') for w in IGNORE_CONJUNCTION_WORDS)
 
 
-def find_dashed_words(text: str) -> Set[str]:
+def find_dashed_words(text: str) -> set[str]:
     dashed_words = [d for d in re.findall(r'\w+- \w+', text) if not is_ignored_by_conjunction_word(d)]
     return dashed_words
 
 
-class SwedishDehyphenatorService:
-    """Dehyphen text"""
-
-    def __init__(
-        self,
-        word_frequency_filename: str,
-        whitelist_filename: str,
-        whitelist_log_filename: str,
-        unresolved_filename: str,
-        word_frequencies: dict = None,
-        whitelist: Set = None,
-        whitelist_log: dict = None,
-        unresolved: Set = None,
-    ):
-
-        self.word_frequency_filename: str = word_frequency_filename
-        self.whitelist_filename: str = whitelist_filename
-        self.whitelist_log_filename: str = whitelist_log_filename
-        self.unresolved_filename: str = unresolved_filename
-
-        if not word_frequencies:
-            if not os.path.isfile(self.word_frequency_filename):
-                raise FileNotFoundError(self.word_frequency_filename)
-
-        # FIXME: Use PersistentDict to load/store dicts
-        self.dehyphenator: SwedishDehyphenator = SwedishDehyphenator(
-            word_frequencies=load_dict(self.word_frequency_filename) if word_frequencies is None else word_frequencies,
-            whitelist=load_token_set(self.whitelist_filename) if whitelist is None else whitelist,
-            whitelist_log=load_dict(self.whitelist_log_filename) if whitelist_log is None else whitelist_log,
-            unresolved=load_token_set(self.unresolved_filename) if unresolved is None else unresolved,
-        )
-
-    def flush(self):
-        store_token_set(self.dehyphenator.whitelist, self.whitelist_filename)
-        store_token_set(self.dehyphenator.unresolved, self.unresolved_filename)
-        store_dict(self.dehyphenator.whitelist_log, self.whitelist_log_filename)
-
-    @staticmethod
-    def create_dehypen(**opts) -> Callable[[str], str]:
-        """Create a dehypen service. Return wrapped dehypen function."""
-        service = SwedishDehyphenatorService(**opts)
-
-        def dehyphen(text: str) -> str:
-            """Remove hyphens from `text`."""
-            dehyphenated_text = service.dehyphenator.dehyphen_text(text)
-
-            return dehyphenated_text
-
-        return dehyphen
-
-
 class ParagraphMergeStrategy(IntEnum):
     DoNotMerge = (0,)
     MergeIfWordsOnlySeparatedByTwoNewlines = (1,)
     MergeAll = 2
 
 
+WORD_FREQUENCIES_NAME: str = 'word-frequencies.pkl'
+
+
 @dataclass
 class SwedishDehyphenator:
     """Dehyphens Swedish text"""
 
-    # External data
-    word_frequencies: Dict[str, int]
+    data_folder: str
+    word_frequencies: dict[str, int] | str
 
     # Internal data
-    whitelist: Set[str] = field(default_factory=set)
-    whitelist_log: Dict[str, int] = field(default_factory=dict)
-    unresolved: Set[str] = field(default_factory=set)
+    whitelist: set[str] = field(default_factory=set)
+    whitelist_log: dict[str, int] = field(default_factory=dict)
+    unresolved: set[str] = field(default_factory=set)
 
     paragraph_merge_strategy: ParagraphMergeStrategy = 0
 
+    word_frequencies_filename: str = field(default=WORD_FREQUENCIES_NAME)
+
+    def __post_init__(self) -> "SwedishDehyphenator":
+        if not isinstance(self.word_frequencies, (dict, type(None))):
+            self.word_frequencies_filename: str = self.resolve_word_frequencies_filename()
+            self.word_frequencies = load_dict(self.word_frequencies_filename)
+
+        self.load()
+
+    def resolve_word_frequencies_filename(self) -> str:
+        if not isinstance(self.word_frequencies, str):
+            if not isfile(self.word_frequencies_filename):
+                return jj(self.data_folder, self.word_frequencies_filename)
+            return self.word_frequencies_filename
+
+        if isfile(self.word_frequencies):
+            return self.word_frequencies
+
+        if isfile(jj(self.data_folder, self.word_frequencies)):
+            return jj(self.data_folder, self.word_frequencies)
+
+        if isfile(jj(self.data_folder, self.word_frequencies_filename)):
+            return jj(self.data_folder, self.word_frequencies_filename)
+
+        if isfile(self.word_frequencies_filename):
+            return self.word_frequencies_filename
+
+        raise FileNotFoundError(f"expected TF file {self.word_frequencies} not found")
+
+    @property
+    def whitelist_filename(self) -> str:
+        return jj(self.data_folder, 'dehyphen_whitelist.txt.gz')
+
+    @property
+    def whitelist_log_filename(self) -> str:
+        return jj(self.data_folder, 'dehyphen_whitelist_log.pkl')
+
+    @property
+    def unresolved_filename(self) -> str:
+        return jj(self.data_folder, 'dehyphen_unresolved.txt.gz')
+
     def is_whitelisted(self, word: str) -> bool:
         return word.lower() in self.whitelist
 
@@ -150,7 +144,7 @@ class SwedishDehyphenator:
     def is_hyphenated_compound(dashed_word: str) -> bool:
         """Test if is compund"""
         if re.match(
-            r'[A-ZÅÄÖ]+-[a-zåäö]+|' r'[A-ZÅÄÖ][a-zåäö]+-[A-ZÅÄÖ][a-zåäö]+|' r'\d+-\w+|' r'icke-\w+',
+            r'[A-ZÅÄÖ]+-[a-zåäö]+|' + r'[A-ZÅÄÖ][a-zåäö]+-[A-ZÅÄÖ][a-zåäö]+|' + r'\d+-\w+|' + r'icke-\w+',
             dashed_word,
         ):
             return True
@@ -162,8 +156,8 @@ class SwedishDehyphenator:
         compound_word: str = re.sub('- ', '', dash)
         dashed_word: str = re.sub('- ', '-', dash)
 
-        _compound_word = compound_word.lower()
-        _dashed_word = dashed_word.lower()
+        _compound_word: str = compound_word.lower()
+        _dashed_word: str = dashed_word.lower()
 
         if self.is_whitelisted(_compound_word):
             return compound_word
@@ -174,8 +168,8 @@ class SwedishDehyphenator:
         if self.is_hyphenated_compound(dashed_word):
             return self.add_to_whitelist(dashed_word, WhitelistReason.HyphenatedCompound)
 
-        _compound_word_frequency = self.word_frequencies.get(_compound_word, 0)
-        _dashed_word_frequency = self.word_frequencies.get(_dashed_word, 0)
+        _compound_word_frequency: int = self.word_frequencies.get(_compound_word, 0)
+        _dashed_word_frequency: int = self.word_frequencies.get(_dashed_word, 0)
 
         if _compound_word_frequency > _dashed_word_frequency:
             return self.add_to_whitelist(compound_word, WhitelistReason.Frequency)
@@ -202,7 +196,7 @@ class SwedishDehyphenator:
 
     def dehyphen_text(self, text: str) -> str:
         """Remove dehyphens in text"""
-        text = re.sub(r'\n{3,}', r'\n\n', text)
+        text: str = re.sub(r'\n{3,}', r'\n\n', text)
 
         # add paragraph markers:
         text = re.sub(r'\n\n', PARAGRAPH_MARKER, text)
@@ -227,6 +221,31 @@ class SwedishDehyphenator:
 
         return text
 
+    def flush(self):
+        store_token_set(self.whitelist, self.whitelist_filename)
+        store_token_set(self.unresolved, self.unresolved_filename)
+        store_dict(self.whitelist_log, self.whitelist_log_filename)
+
+    def load(self) -> None:
+        self.whitelist = load_token_set(self.whitelist_filename)
+        self.whitelist_log = load_dict(self.whitelist_log_filename)
+        self.unresolved = load_token_set(self.unresolved_filename)
+
+    @staticmethod
+    def create_dehypen(data_folder: str, word_frequencies: str | dict = None) -> Callable[[str], str]:
+        """Create a dehyphen service. Return wrapped dehyphen function."""
+        dehyphenator: SwedishDehyphenator = SwedishDehyphenator(
+            data_folder=data_folder, word_frequencies=word_frequencies
+        )
+
+        def dehyphen(text: str) -> str:
+            """Remove hyphens from `text`."""
+            dehyphenated_text = dehyphenator.dehyphen_text(text)
+
+            return dehyphenated_text
+
+        return dehyphen
+
 
 def merge_paragraphs(text: str, paragraph_merge_strategy: ParagraphMergeStrategy) -> str:
     """Merge paragraphs"""
